unit DDay.Form.Main;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.Samples.Spin,
  System.Rtti, System.Diagnostics, Vcl.Imaging.pngimage, Vcl.ExtCtrls;

type
  TfrmMain = class(TForm)
    btnQueryPerformance: TButton;
    memoLog: TMemo;
    btnTickCount: TButton;
    edtSleep: TSpinEdit;
    Label1: TLabel;
    btnStopWatch: TButton;
    btnNow: TButton;
    btnTickCount64: TButton;
    imgClear: TImage;
    procedure btnNowClick(Sender: TObject);
    procedure btnQueryPerformanceClick(Sender: TObject);
    procedure btnStopWatchClick(Sender: TObject);
    procedure btnTickCount64Click(Sender: TObject);
    procedure btnTickCountClick(Sender: TObject);
    procedure imgClearClick(Sender: TObject);
  private
    procedure Log(const AValue: TValue);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  System.DateUtils;

{$R *.dfm}

procedure TfrmMain.btnNowClick(Sender: TObject);
var
  LStart, LStop: TDateTime;
  LElapsed: Int64;
begin
  LStart := Now;
  Sleep(edtSleep.Value);
  LStop := Now;
  LElapsed := MilliSecondsBetween(LStop, LStart);

  Log(LElapsed);
end;

procedure TfrmMain.btnQueryPerformanceClick(Sender: TObject);
var
  LFrequency, LStartCount, LStopCount: Int64;
  LElapsed: Double;
begin
  QueryPerformanceFrequency(LFrequency);
  QueryPerformanceCounter(LStartCount);
  Sleep(edtSleep.Value);
  QueryPerformanceCounter(LStopCount);
  LElapsed := 1000 * ((LStopCount - LStartCount) / LFrequency);

  Log(LElapsed);
end;

procedure TfrmMain.btnStopWatchClick(Sender: TObject);
var
  LStopWatch: TStopWatch;
  LElapsed: Int64;
begin
  LStopWatch := TStopWatch.Create();
  LStopWatch.Start;
  Sleep(edtSleep.Value);
  LStopWatch.Stop;
  LElapsed := LStopWatch.ElapsedMilliseconds;

  Log(LElapsed);
end;

procedure TfrmMain.btnTickCount64Click(Sender: TObject);
var
  LStartCount, LStopCount: UInt64;
  LElapsed: UInt64;
begin
  LStartCount := GetTickCount64();
  Sleep(edtSleep.Value);
  LStopCount := GetTickCount64();
  LElapsed := LStopCount - LStartCount;

  Log(LElapsed);
end;

procedure TfrmMain.btnTickCountClick(Sender: TObject);
var
  LStartCount, LStopCount: Cardinal;
  LElapsed: Cardinal;
begin
  LStartCount := GetTickCount();
  Sleep(edtSleep.Value);
  LStopCount := GetTickCount();
  LElapsed := LStopCount - LStartCount;

  Log(LElapsed);
end;

procedure TfrmMain.imgClearClick(Sender: TObject);
begin
  memoLog.Lines.Clear;
end;

procedure TfrmMain.Log(const AValue: TValue);
begin
  memoLog.Lines.Add(Format('Sleep interval of %d ms, measured %s ms', [edtSleep.Value, AValue.ToString]));
end;

end.
